@@ -33,7 +33,6 @@
 #include <boost/optional/optional.hpp>
 #include <string>
 #include "net_utils_base.h"
-#include "to_nonconst_iterator.h"
 #include "http_auth.h"
 #include "http_base.h"
 
@@ -56,6 +55,7 @@ namespace net_utils
 			std::string m_folder;
 			std::vector<std::string> m_access_control_origins;
 			boost::optional<login> m_user;
+			size_t m_max_content_length{std::numeric_limits<size_t>::max()};
 			critical_section m_lock;
 		};
 
@@ -142,6 +142,7 @@ namespace net_utils
 			config_type& m_config;
 			bool m_want_close;
 			size_t m_newlines;
+			size_t m_bytes_read;
 		protected:
 			i_service_endpoint* m_psnd_hndlr; 
 			t_connection_context& m_conn_context;
