@@ -1,4 +1,4 @@
-// Copyright (c) 2019-2020, The Monero Project
+// Copyright (c) 2019-2022, The Monero Project
 // 
 // All rights reserved.
 // 
@@ -38,10 +38,10 @@ inline bool do_serialize(Archive<false>& ar, cryptonote::difficulty_type &diff)
 {
   uint64_t hi, lo;
   ar.serialize_varint(hi);
-  if (!ar.stream().good())
+  if (!ar.good())
     return false;
   ar.serialize_varint(lo);
-  if (!ar.stream().good())
+  if (!ar.good())
     return false;
   diff = hi;
   diff <<= 64;
@@ -52,13 +52,13 @@ inline bool do_serialize(Archive<false>& ar, cryptonote::difficulty_type &diff)
 template <template <bool> class Archive>
 inline bool do_serialize(Archive<true>& ar, cryptonote::difficulty_type &diff)
 {
-  if (!ar.stream().good())
+  if (!ar.good())
     return false;
   const uint64_t hi = ((diff >> 64) & 0xffffffffffffffff).convert_to<uint64_t>();
   const uint64_t lo = (diff & 0xffffffffffffffff).convert_to<uint64_t>();
   ar.serialize_varint(hi);
   ar.serialize_varint(lo);
-  if (!ar.stream().good())
+  if (!ar.good())
     return false;
   return true;
 }
